import { isVoid } from "./type-system";

export function registerXsrfHeaderInAjaxPost() {
    $(document).ajaxSend(function (event, jqxhr, settings) {
        if (settings.type.toUpperCase() !== 'POST') return;
        jqxhr.setRequestHeader(_.xsrfHeaderName(), _.xsrfValue);
    });
}
export function createUrl(url) {
    var culture = _.culture();
    var u = url.startsWith('/') ? url.substr(1) : url;
    var res;
    if (_.cultures().length > 0) {
        res = `/${culture}/${u}`;
    } else {
        res = `/${u}`;
    }
    return res;
}
export function createAjaxOptions(method, url, data, success, error, completed) {
    var res = {
        method: method,
        dataType: 'text',
        url: url,
        data: data,
        cache: false,
        complete: function (jqXHR, status) {
            if (status === 'success') {
                if (!isVoid(success)) {
                    var json = JSON.parse(jqXHR.responseText);
                    if (json.success) {
                        success(json.returnValue, json);
                    } else {
                        if (!isVoid(error)) {
                            error(json);
                        }
                    }
                }
            } else {
                if (!isVoid(error)) {
                    error(status);
                }
            }
            if (!isVoid(completed)) {
                completed(jqXHR, status);
            }
        },
    };
    if (data instanceof FormData) {
        res.processData = false;
        res.contentType = false;
    }
    return res;
}
export function send(method, url, data, success, error, completed) {
    return $.ajax(createAjaxOptions(method, url, data, success, error, completed));
}
export function sendGet(url, data, success, error, completed) {
    return send('GET', url, data, success, error, completed);
}
export function sendPost(url, data, success, error, completed) {
    return send('POST', url, data, success, error, completed);
}