import { dateTimeFormats } from './dateTimeFormats';
import { calendars } from './calendars';
import { isVoid } from './type-system';
export function dat(cal, unix_milliseconds) {
    var calendar = isVoid(cal) ? _.calendar() : cal;
    var value = isVoid(unix_milliseconds) ? calendar() : calendar(unix_milliseconds);
    var getMethodName = x => {
        var methodName = x;
        if (calendar === calendars.jalaali) {
            methodName = "j" + methodName[0].toUpperCase() + methodName.substr(1);
        } else if (calendar === calendars.hijri) {
            methodName = "i" + methodName[0].toUpperCase() + methodName.substr(1);
        }
        return methodName;
    };
    var v = () => {
        return calendar(value.valueOf());
    };
    this.year = year => {
        var methodName = getMethodName("year");
        if (isVoid(year)) {
            return value[methodName]();
        }
        var k = v()[methodName](year);
        return new dat(calendar, k.valueOf());
    };
    this.month = month => {
        var methodName = getMethodName("month");
        if (isVoid(month)) {
            return value[methodName]() + 1;
        }
        var k = v()[methodName](month - 1);
        return new dat(calendar, k.valueOf());
    };
    this.day = day => {
        if (isVoid(day)) {
            return value.date();
        }
        var k = v().date(day);
        return new dat(calendar, k.valueOf());
    };
    this.hour = hour => {
        if (isVoid(hour)) {
            return value.hour();
        }
        var k = v().hour(hour);
        return new dat(calendar, k.valueOf());
    };
    this.minute = minute => {
        if (isVoid(minute)) {
            return value.minute();
        }
        var k = v().minute(minute);
        return new dat(calendar, k.valueOf());
    };
    this.second = second => {
        if (isVoid(second)) {
            return value.second();
        }
        var k = v().second(second);
        return new dat(calendar, k.valueOf());
    };
    this.millisecond = millisecond => {
        if (isVoid(millisecond)) {
            return value.millisecond();
        }
        var k = v().millisecond(millisecond);
        return new dat(calendar, k.valueOf());
    };
    this.unixMilliseconds = unix_milli => {
        if (isVoid(unix_milli)) {
            return value.valueOf();
        }
        var k = calendar(unix_milli);
        return new dat(calendar, k.valueOf());
    };
    this.unixTicks = unix_ticks => {
        if (isVoid(unix_ticks)) {
            return value.valueOf() * 10000;
        }
        var k = calendar(unix_ticks / 10000);
        return new dat(calendar, k.valueOf());
    };
    this.withCalendar = c => {
        return new dat(c, value.valueOf());
    };
    this.format = fmt => {
        if (fmt === dateTimeFormats.all) {
            return String.format(
                "{0}/{1}/{2} {3}:{4}:{5}.{6}",
                this.year().padZero(4),
                this.month().padZero(2),
                this.day().padZero(2),
                this.hour().padZero(2),
                this.minute().padZero(2),
                this.second().padZero(2),
                this.millisecond()
            );
        }
        if (fmt === dateTimeFormats.ymdhms) {
            return String.format(
                "{0}/{1}/{2} {3}:{4}:{5}",
                this.year().padZero(4),
                this.month().padZero(2),
                this.day().padZero(2),
                this.hour().padZero(2),
                this.minute().padZero(2),
                this.second().padZero(2)
            );
        }
        if (fmt === dateTimeFormats.ymdhm) {
            return String.format(
                "{0}/{1}/{2} {3}:{4}",
                this.year().padZero(4),
                this.month().padZero(2),
                this.day().padZero(2),
                this.hour().padZero(2),
                this.minute().padZero(2)
            );
        }
        if (fmt === dateTimeFormats.ymdh) {
            return String.format(
                "{0}/{1}/{2} {3}",
                this.year().padZero(4),
                this.month().padZero(2),
                this.day().padZero(2),
                this.hour().padZero(2)
            );
        }
        if (fmt === dateTimeFormats.ymd) {
            return String.format(
                "{0}/{1}/{2}",
                this.year().padZero(4),
                this.month().padZero(2),
                this.day().padZero(2)
            );
        }
        if (fmt === dateTimeFormats.ym) {
            return String.format(
                "{0}/{1}",
                this.year().padZero(4),
                this.month().padZero(2)
            );
        }
        if (fmt === dateTimeFormats.y) {
            return String.format("{0}", this.year().padZero(4));
        }
        if (fmt === dateTimeFormats.hmsm) {
            return String.format(
                "{0}:{1}:{2}.{3}",
                this.hour().padZero(2),
                this.minute().padZero(2),
                this.second().padZero(2),
                this.millisecond()
            );
        }
        if (fmt === dateTimeFormats.hms) {
            return String.format(
                "{0}:{1}:{2}",
                this.hour().padZero(2),
                this.minute().padZero(2),
                this.second().padZero(2)
            );
        }
        if (fmt === dateTimeFormats.hm) {
            return String.format(
                "{0}:{1}",
                this.hour().padZero(2),
                this.minute().padZero(2)
            );
        }
        if (fmt === dateTimeFormats.h) {
            return String.format("{0}", this.hour().padZero(2));
        }
    };
    /**
     * @param {String} methodName: year, month, day, hour, minute, second
     */
    this.startOf = methodName => {
        var x = methodName;
        if (methodName.toLowerCase() === "year" || methodName.toLowerCase() === "month") {
            x = getMethodName(x);
        }
        var k = v().startOf(x);
        return new dat(calendar, k.valueOf());
    };
    /**
     * @param {String} methodName: year, month, day, hour, minute, second
     */
    this.endOf = methodName => {
        var x = methodName;
        if (methodName.toLowerCase() === "year" || methodName.toLowerCase() === "month") {
            x = getMethodName(x);
        }
        var k = v().endOf(x);
        return new dat(calendar, k.valueOf());
    };
    /**
     * @param {Number} val
     * @param {String} methodName: year, month, day, hour, minute, second, millisecond
     */
    this.add = (val, methodName) => {
        var x = methodName;
        if (methodName.toLowerCase() === "year" || methodName.toLowerCase() === "month") {
            x = getMethodName(x);
        }
        var k = v().add(val, x);
        return new dat(calendar, k.valueOf());
    };
    /**
     * @param {Number} val
     * @param {String} methodName: year, month, day, hour, minute, second, millisecond
     */
    this.subtract = (val, methodName) => {
        var x = methodName;
        if (methodName.toLowerCase() === "year" || methodName.toLowerCase() === "month") {
            x = getMethodName(x);
        }
        var k = v().subtract(val, x);
        return new dat(calendar, k.valueOf());
    };
    this.daysInMonth = (month, year) => {
        var methodName = getMethodName("daysInMonth");
        var m = isVoid(month) ? this.month() - 1 : month - 1;
        var y = isVoid(year) ? this.year() : year;
        return calendar[methodName](y, m);
    };
}